package de.rico.adventure.structure;

import de.rico.engine.geometry.Point3D;

/**
 * This class encapsulates a location of entry in a room. Such a
 * location is specified by position and rotation values.
 * @author Frank Bruns
 *
 */
public class EntryPoint
{
	private String scriptId = null;
	private Point3D position = null;
	private Point3D rotation = null;

	/**
	 * Creates a new entry point with the specified position
	 * and rotation.
	 * @param scriptId script id of the entry point
	 * @param pos position of the entry point
	 * @param rot rotation at the entry point
	 */
	public EntryPoint(String scriptId, Point3D pos, Point3D rot)
	{
		this.scriptId = scriptId;
		this.position = pos;
		this.rotation = rot;
	}

	/**
	 * Returns the script id of the entry point.
	 * @return entry point's script id
	 */
	public String getScriptId()
	{
		return this.scriptId;
	}

	/**
	 * Returns the position of the entry point.
	 * @return positon of the entry point
	 */
	public Point3D getPosition()
	{
		return this.position;
	}

	/**
	 * Returns the rotation of the entry point.
	 * @return rotation of the entry point
	 */
	public Point3D getRotation()
	{
		return this.rotation;
	}
}
