/**
 *
 */
package de.rico.editor.datamodel;


/**
 * Data model class of an entity for use in the editor.
 * @author Frank Bruns
 */
public abstract class AbstractEntityData
{
	private MeshData mesh = null;
	private String name = "Give me a name, please!";
	private String scriptId = null;

	public AbstractEntityData(MeshData mesh, String name, String scriptId)
	{
		this.mesh = mesh;
		this.name = name;
		this.scriptId = scriptId;
	}

	/**
	 * Returns the entity's name.
	 *
	 * @return name of the entity
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * Returns the mesh that is used for this entity.
	 *
	 * @return the mesh for the entity
	 */
	public MeshData getMesh()
	{
		return this.mesh;
	}

	/**
	 * Returns the reference id for this entity which is used in a script file.
	 *
	 * @return reference id in a script file
	 * @throws NullPointerException Occurs when no id has been specified for the
	 *         entity.
	 */
	public String getScriptId()
	{
		return this.scriptId;
	}

}
