/**
 *
 */
package de.rico.editor.datamodel;

import de.rico.engine.material.Color4f;

/**
 * This class is the abstract base class for lights in the
 * world editor data model.
 * @author Frank Bruns
 */
public class AbstractLightData
{
	private int lightNr = 0;

	private Color4f ambient = new Color4f(0.0f, 0.0f, 0.0f, 1f);

	private Color4f diffuse = new Color4f(1.0f, 1.0f, 1.0f, 1f);

	public AbstractLightData(Color4f amb, Color4f diff, int lightNr)
	{
		this.lightNr = lightNr;
		this.ambient.setColor(amb);
		this.diffuse.setColor(diff);
	}

	/**
	 * Returns the number of this light (0-7).
	 * @return light number
	 */
	public int getLightNr()
	{
		return this.lightNr;
	}

	/**
	 * Returns ambient color.
	 * @return ambient color
	 */
	public Color4f getAmb()
	{
		return this.ambient;
	}

	/**
	 * Returns diffuse color.
	 * @return diffuse color
	 */
	public Color4f getDiff()
	{
		return this.diffuse;
	}
}
