package de.rico.editor.datamodel;

import de.rico.engine.enums.BoundingVolume;
import de.rico.engine.geometry.Point3D;

/**
 * @author Frank Bruns
 */
public class GroundPlaneData extends MeshData
{
	private int size = 0;
	private float texRepeat = 0;
	private int gridNodeSize = 0;

	public GroundPlaneData(String name, String tex, Point3D pos, Point3D rot, int size, float texRep, int gridNodeSize, boolean rotate, boolean flip, boolean smooth)
	{
		super(null, name, tex, pos, rot, rotate, flip, smooth,false,BoundingVolume.BOX);
		this.size = size;
		this.texRepeat = texRep;
		this.gridNodeSize = gridNodeSize;
	}

	/**
	 * Returns the size of the ground plane.
	 * @return size of the ground plane
	 */
	public int getSize()
	{
		return this.size;
	}

	/**
	 * Returns the texture repeat factor of the ground plane.
	 * @return texture repeat factor of the ground plane
	 */
	public float getTexRepeat()
	{
		return this.texRepeat;
	}

	/**
	 * Return the grid node size.
	 * @return grid node size
	 */
	public int getGridNodeSize()
	{
		return this.gridNodeSize;
	}
}
