/**
 *
 */
package de.rico.editor.datamodel;

import de.rico.engine.geometry.Point3D;
import de.rico.engine.material.Color4f;

/**
 * This class represents a spot light in the world editor' data model.
 * @author Frank Bruns
 */
public class SpotLightData extends PosLightData
{
	private float rotX = 0;
	private float rotY = 0;
	private float cutOff = 0;
	private float exponent = 0;

	public SpotLightData(Point3D pos, float rotX, float rotY, Color4f amb, Color4f diff, float cutOff, float exp, int lightNr)
	{
		super(pos,amb,diff,lightNr);
		this.rotX = rotX;
		this.rotY = rotY;
		this.cutOff = cutOff;
		this.exponent = exp;
	}

	/**
	 * Returns the spot light's rotation around the x-axis.
	 * @return rotation around the x-axis
	 */
	public float getRotX()
	{
		return this.rotX;
	}

	/**
	 * Returns the spot light's rotation around the y-axis.
	 * @return rotation around the y-axis
	 */
	public float getRotY()
	{
		return this.rotX;
	}

	/**
	 * Returns the spot cutoff.
	 * @return spot cutOff
	 */
	public float getCutOff()
	{
		return this.cutOff;
	}

	/**
	 * Returns the spot exponent.
	 * @return spot exponent
	 */
	public float getExponent()
	{
		return this.exponent;
	}
}
